#
#	general.link.mk
#
#	SNiFF+ V3.0
#
#	Common make rules for making executable, relinkable object and library.
#
#	Copyright (C) 1998 TakeFive Software GmbH
#	Salzburg, Austria (e-mail: sniff@TakeFive.co.at)
#
#	Author: Sandor Korosi

LINK_OUTPUT_FLAG		= -o $@
RELINK_OUTPUT_FLAG		= -o $@
LIB_OUTPUT_FLAG			= $@
SHAREDLIB_OUTPUT_FLAG	= -o $@
 
#       All linker flags
ALL_LDFLAGS = $(OTHER_LDFLAGS) $(COMMON_LDFLAGS) $(OS_LDFLAGS) $(LDFLAGS) 
#       Command to link an executable
LINK_CMD = $(LINK) $(ALL_LDFLAGS) $(LINK_OUTPUT_FLAG) $(OFILES) $(OTHER_OFILES) $(SUB_RELINK_OFILES) $(SUB_LIBS) $(SUB_LIBS2) $(SNIFF_SUB_SHARED_LIBS_LINK) $(OTHER_LIBS) $(OS_LIBS) $(SPECIAL_LINK_FLAGS)

#       Command to make a relocatable object
RELINK_CMD = $(LD) $(RELINK_OUTPUT_FLAG) $(OFILES) $(OTHER_OFILES) $(SUB_RELINK_OFILES) $(SUB_LIBS) $(RELINK_OTHER_LIBS)

#       Command to make a library
LIB_CMD = $(AR) $(ARFLAGS) $(LIB_OUTPUT_FLAG) $(OFILES) $(OTHER_OFILES) $(SUB_RELINK_OFILES)

#       Command to make a shared library
SHAREDLIB_CMD = $(LD_SHARED) $(LD_SHAREDFLAGS) $(SHAREDLIB_OUTPUT_FLAG) $(OFILES) $(OTHER_OFILES) $(SUB_RELINK_OFILES) $(LD_SHAREDFLAGS_MSWIN)

#       All purify flags
ALL_PURIFYFLAGS = $(OTHER_PURIFYFLAGS) $(COMMON_PURIFYFLAGS) $(OS_PURIFYFLAGS) $(PURIFYFLAGS)
#       Command to make a purified executable
PURIFY_CMD = $(PURIFY) $(ALL_PURIFYFLAGS)

#       All quantify flags
ALL_QUANTIFYFLAGS = $(OTHER_QUANTIFYFLAGS) $(COMMON_QUANTIFYFLAGS) $(OS_QUANTIFYFLAGS) $(QUANTIFYFLAGS)
#       Command to make a quantified executable
QUANTIFY_CMD = $(QUANTIFY) $(ALL_QUANTIFYFLAGS)

#       target to link executable
$(TARGET_DIR)/$(LINK_TARGET): check_dirs $(PRE_TARGETS) $(OFILES) $(OTHER_OFILES) $(SUB_RELINK_OFILES) $(SUB_LIBS) $(SNIFF_SUB_SHARED_LIBS) $(DEBUG_LIBS) $(FORCE_RULE)
	@ d=`dirname '$@'`;\
	relative=`echo $$d | cut -d"/" -f1`;\
	if [ "$$d" != "." -a "$$d" != "$(TARGET_DIR)" -a "$$relative" != "" ]; then\
		cd $$d;\
		echo make: Changing directory to \'$$d\';\
		$(OWN_MAKE) -f $(MAKEFILE) `basename '$@'`;\
	else\
		TARGET=$@; TARGET_TYPE="executable"; $(MAKE_TARGET);\
		if [ "$(DEBUG)" = "1" ]; then \
			echo "$(LINK_CMD) $(DEBUG_LIBS)";\
		fi;\
		$(LINK_CMD) $(DEBUG_LIBS);\
		ret=$$?;\
		if [ "$(EXE_SUFFIX)" != "" -a $$ret = 0 ]; then\
			echo $@ | grep -i \\$(EXE_SUFFIX) >$(DEVNULL);\
			if [ $$? != 0 ]; then\
				$(MV) $@ $@.$(EXE_SUFFIX);\
				echo "" > $@;\
			fi;\
		fi;\
		exit $$ret;\
	fi;

#       target to make a relocatable object
$(TARGET_DIR)/$(RELINK_TARGET): $(PRE_TARGETS) $(OFILES) $(OTHER_OFILES) $(SUB_RELINK_OFILES) $(SUB_LIBS) $(FORCE_RULE)
	@ d=`dirname '$@'`;\
	relative=`echo $$d | cut -d"/" -f1`;\
	if [ "$$d" != "." -a "$$d" != "$(TARGET_DIR)" -a "$$relative" != "" ]; then\
		cd $$d;\
		echo make: Changing directory to \'$$d\';\
		$(OWN_MAKE) -f $(MAKEFILE) `basename '$@'`;\
	else\
		TARGET=$@; TARGET_TYPE="relinkable object"; $(MAKE_TARGET);\
		if [ "$(DEBUG)" = "1" ]; then \
			echo "$(RELINK_CMD)";\
		fi;\
		$(RELINK_CMD);\
	fi;

#       target to make library
$(TARGET_DIR)/$(LIB_TARGET): $(PRE_TARGETS) $(OFILES) $(OTHER_OFILES) $(SUB_RELINK_OFILES) $(FORCE_RULE)
	@ d=`dirname '$@'`;\
	relative=`echo $$d | cut -d"/" -f1`;\
	if [ "$$d" != "." -a "$$d" != "$(TARGET_DIR)" -a "$$relative" != "" ]; then\
		cd $$d;\
		echo make: Changing directory to \'$$d\';\
		$(OWN_MAKE) -f $(MAKEFILE) `basename '$@'`;\
	else\
		$(RM) $@; \
		TARGET=$@; TARGET_TYPE="library"; $(MAKE_TARGET);\
		if [ "$(DEBUG)" = "1" ]; then \
			echo "$(LIB_CMD)";\
		fi;\
		$(LIB_CMD);\
		if [ "$(RANLIB)" != "true" ]; then\
			echo "make: Making ranlib $@ ...";\
			if [ "$(DEBUG)" = "1" ]; then \
				echo "$(RANLIB) $@";\
			fi;\
			$(RANLIB) $@;\
		fi;\
	fi;

#       target to make shared library
$(TARGET_DIR)/$(SHAREDLIB_TARGET): $(PRE_TARGETS) $(OFILES) $(OTHER_OFILES) $(SUB_RELINK_OFILES) $(FORCE_RULE)
	@ d=`dirname '$@'`;\
	relative=`echo $$d | cut -d"/" -f1`;\
	if [ "$$d" != "." -a "$$d" != "$(TARGET_DIR)" -a "$$relative" != "" ]; then\
		cd $$d;\
		echo make: Changing directory to \'$$d\';\
		$(OWN_MAKE) -f $(MAKEFILE) `basename '$@'`;\
	else\
		$(RM) $@; \
		TARGET=$@; TARGET_TYPE="shared library"; $(MAKE_TARGET);\
		if [ "$(DEBUG)" = "1" ]; then \
			echo "$(SHAREDLIB_CMD)";\
		fi;\
		$(SHAREDLIB_CMD);\
	fi;

#       target to link executable with purify
$(TARGET_DIR)/$(PURIFY_TARGET): $(PRE_TARGETS) $(OFILES) $(OTHER_OFILES) $(SUB_RELINK_OFILES) $(SUB_LIBS) $(SNIFF_SUB_SHARED_LIBS) $(DEBUG_LIBS) $(FORCE_RULE)
	@ d=`dirname '$@'`;\
	relative=`echo $$d | cut -d"/" -f1`;\
	if [ "$$d" != "." -a "$$d" != "$(TARGET_DIR)" -a "$$relative" != "" ]; then\
		cd $$d;\
		echo make: Changing directory to \'$$d\';\
		$(OWN_MAKE) -f $(MAKEFILE) `basename '$@'`;\
	else\
		TARGET=$@; TARGET_TYPE="purified executable"; $(MAKE_TARGET);\
		if [ "$(DEBUG)" = "1" ]; then \
			echo "$(PURIFY_CMD) $(LINK_CMD) $(DEBUG_LIBS)";\
		fi;\
		$(PURIFY_CMD) $(LINK_CMD) $(DEBUG_LIBS);\
	fi;

#       target to link executable with quantify
$(TARGET_DIR)/$(QUANTIFY_TARGET):	$(PRE_TARGETS) $(OFILES) $(OTHER_OFILES) $(SUB_RELINK_OFILES) $(SUB_LIBS) $(SNIFF_SUB_SHARED_LIBS) $(FORCE_RULE)
	@ d=`dirname '$@'`;\
	relative=`echo $$d | cut -d"/" -f1`;\
	if [ "$$d" != "." -a "$$d" != "$(TARGET_DIR)" -a "$$relative" != "" ]; then\
		cd $$d;\
		echo make: Changing directory to \'$$d\';\
		$(OWN_MAKE) -f $(MAKEFILE) `basename '$@'`;\
	else\
		TARGET=$@; TARGET_TYPE="quantified executable"; $(MAKE_TARGET);\
		if [ "$(DEBUG)" = "1" ]; then \
			echo "$(QUANTIFY_CMD) $(LINK_CMD)";\
		fi;\
		$(QUANTIFY_CMD) $(LINK_CMD);\
	fi;
