#
#	general.mk
#
#	SNiFF+ V3.0
#
#	Common makefile definitions for all platforms to be included
#	in each project makefile.
#
#	Copyright (C) 1998 TakeFive Software GmbH
#	Salzburg, Austria (e-mail: sniff@TakeFive.co.at)
#
#	Author: Sandor Korosi
#
#	Command line parameters:
#       CHECK_UPDATE = 0    don't check the up-to-date status.
#                           Default: 1, check the up-to-date status
#       DEBUG        = 1    print commands.
#                           Default: 0, don't print
#       FORCE        = 1    enforcement flag.
#                           Default: 0
#       JAVA_GEN_SCRIPT = 0 don't create executable java target scripts.
#                           Default: 1, create
 
MAKEFILE = Makefile

#        Files that are affected by cleanup
CLEANUP = core $(OFILES) $(OTHER_OFILES) $(SNIFF_DERIVED_FILES) $(SNIFF_TARGETS) $(OTHER_CLEANUP)

# The following macros should be used as command line parameters of the make command
#
#        Set default values of the command line parameters
CHECK_UPDATE = 1
DEBUG = 1

#        Make Session macros
SESSION = 0
SESSIONFILE=.Sniff_LastMake
 
#        Directory of the symbol information files
SNIFF_SYMBOL_DIR = $(SNIFF_RELATIVE_ROOT_DIR)/$(SNIFF_GENERATE_DIR_ROOT)/$(SNIFF_PROJECT_DIR)/$(SNIFF_GENERATE_DIR)
 
#        Shell script to make target and vpath in shared working environment environment
MAKE_TARGET	= \
			if [ -h $$TARGET ]; then \
				echo "make: Removing symbolic link to shared $$TARGET"; \
			fi; \
			$(RM) $$TARGET; \
			echo "make: Making $$TARGET_TYPE $$TARGET ..."

PARENT = ./

# suppress automatic retrieval
.SCCS_GET:
.SCCS_GET_POSIX:

.SUFFIXES:

########################################################################
# rule for make default target recursively
########################################################################

all :: subdirs_all main_all

#		 Build Targets always
FORCE_RULE = $(FORCE:1=force)
force: ;

# recursive make in SUBDIRS
subdirs_all :
	@if [ $(DEBUG) != 0 -a "$(SUBDIRS)" != "" ]; then\
		echo "make: starting recursive make for default target in project '$(SNIFF_PROJECT)'";\
		echo "make: $(OWN_MAKE) -f $(MAKEFILE) $(MFLAGS)";\
	fi;\
	_PWD=`pwd`;\
	if [ $(SESSION) = 0 ]; then\
		session=`date +%D-%T`;\
	else\
		session=$(SESSION);\
	fi;\
	$(RM) $(SESSIONFILE);\
	echo $$session > $(SESSIONFILE);\
	for dir in _dummy_ $(SUBDIRS); do\
		if [ $$dir = "_dummy_" ]; then\
			continue ;\
		fi;\
		relative=`echo $$dir | cut -d"/" -f1 | grep -v ":"`;\
		if [ "$$relative" != "" ]; then\
			absdir=$$_PWD/$$dir;\
			parent=$(PARENT)$$dir/; \
		else\
			absdir=$$dir;\
			parent=$$dir/ ;\
		fi;\
		if [ -d $$absdir ]; then\
			cd $$absdir;\
			if [ -f $(SESSIONFILE) ]; then\
				lastSession=`cat $(SESSIONFILE) 2>$(DEVNULL)`;\
			else\
				lastSession=0;\
			fi;\
			if [ "$$lastSession" = "$$session" ]; then\
				echo "*** $$parent already processed";\
			else\
				$(OWN_MAKE) -f $(MAKEFILE) PARENT=$$parent SESSION="$$session" all || exit;\
			fi;\
		else\
			echo "make: project directory $(PARENT)$$dir does not exist. Please create it with SNiFF+ if you need it" ;\
		fi;\
	done;\
	cd "$$_PWD"

# start a new make process to make default target in this project
main_all :
	@ echo "*** $(PARENT)";\
	echo "'$(OWN_MAKE)' -f '$(MAKEFILE)' _all";\
	$(OWN_MAKE) -f $(MAKEFILE) _all

# make default target in this project
_all : check_up-to-date_status check_dirs $(PRE_TARGETS) $(SNIFF_TARGET)
	@if [ $(DEBUG) != 0 ]; then\
		echo "make: default target is done in project '$(SNIFF_PROJECT)'";\
	fi

########################################################################
# rule for make help targets recursively
########################################################################
clean clean_state clean_backup clean_targets clean_objects sniffclean all_pre_targets symbolic_links clean_symbolic_links symbolic_links_to_source_files symbolic_link_to_dependencies_file rmi_all jni_all ::
	@if [ $(DEBUG) != 0 -a "$(SUBDIRS)" != "" ]; then\
		echo "make: starting recursive make for help target in project '$(SNIFF_PROJECT)'";\
		echo "make: $(OWN_MAKE) -f $(MAKEFILE) $(MFLAGS) '$@'";\
	fi;\
	_PWD=`pwd`;\
	if [ $(SESSION) = 0 ]; then\
		session=`date +%D-%T`;\
	else\
		session=$(SESSION);\
	fi;\
	$(RM) $(SESSIONFILE);\
	echo $$session > $(SESSIONFILE);\
	for dir in _dummy_ $(SUBDIRS); do\
		if [ $$dir = "_dummy_" ]; then\
			continue ;\
		fi;\
		relative=`echo $$dir | cut -d"/" -f1 | grep -v ":"`;\
		if [ "$$relative" != "" ]; then\
			absdir=$$_PWD/$$dir;\
			parent=$(PARENT)$$dir/; \
		else\
			absdir=$$dir;\
			parent=$$dir/ ;\
		fi;\
		if [ -d $$absdir ]; then\
			cd $$absdir;\
			if [ -f $(SESSIONFILE) ]; then\
				lastSession=`cat $(SESSIONFILE) 2>$(DEVNULL)`;\
			else\
				lastSession=0;\
			fi;\
			if [ "$$lastSession" = "$$session" ]; then\
				echo "*** $$parent already processed";\
			else\
				$(OWN_MAKE) -f $(MAKEFILE) PARENT=$$parent SESSION="$$session" '$@' || exit;\
			fi;\
		else\
			echo "make: project directory $(PARENT)$$dir does not exist. Please create it with SNiFF+ if you need it" ;\
		fi;\
	done;\
	exit 0

########################################################################
# check_dirs
#	check_dirs for object and target files
########################################################################
check_dirs :: _check_dirs
_check_dirs:
	@ if [ ! -d $(OBJECT_DIR) ]; then \
		echo "make: Creating Directory for Object Files"; \
		$(MK_DIR) $(OBJECT_DIR); \
	fi; \
	if [ ! -d $(TARGET_DIR) ]; then \
		echo "make: Creating Directory for Target Files"; \
		$(MK_DIR) $(TARGET_DIR); \
	fi
 
########################################################################
# check_up-to-date_status
#	check the up-to-date status of project directory
########################################################################
check_up-to-date_status :: _check_up-to-date_status
_check_up-to-date_status :
	@ if [ "$(CHECK_UPDATE)" = "1" ]; then\
		if [ $(DEBUG) != 0 ]; then\
			echo "make: Checking up-to-date status of project";\
		fi;\
		for dir in `echo "$(SNIFF_ShSWS) $(SNIFF_ShOWS)" |sed -e "s/$(D)/ /g"`; do\
			if [ -r "$$dir/$(SNIFF_STATUSFILE)" ]; then\
				if [ ! -r "./$(SNIFF_STATUSFILE)" ]; then\
					if [ $(DEBUG) = 0 ]; then\
						echo "make: Checking up-to-date status of project";\
					fi;\
					echo "make: Please either execute 'Synchronize checkmarked Projects' to update your working environment";\
					echo "      or start build again with the flag CHECK_UPDATE=0. This flag forces SNiFF+ to";\
					echo "      ignore the update status of your working environment";\
					exit 1;\
				fi;\
				if [ "`find $$dir/$(SNIFF_STATUSFILE) -name $(SNIFF_STATUSFILE) -newer ./$(SNIFF_STATUSFILE) -print`" != "" ]; then\
					if [ $(DEBUG) = 0 ]; then\
						echo "make: Checking up-to-date status of project";\
					fi;\
					echo "make: The update status of the shared working environment $$dir";\
					echo "      is newer than the update status of your working environment.";\
					echo "      This means that source files in your working environment";\
					echo "      may not be up-to-date.";\
					echo "      Please either execute 'Synchronize checkmarked Projects' to update your working environment";\
					echo "      or start build again with the flag CHECK_UPDATE=0. This flag forces SNiFF+ to";\
					echo "      ignore the update status of your working environment";\
					exit 1;\
				fi;\
			fi;\
		done;\
	fi
 
 
########################################################################
# pre_targets
#	make pre-targets
# all_pre_targets
#	make pre-targets recursively
########################################################################
pre_targets :: _all_pre_targets

all_pre_targets :: echo_parent _all_pre_targets
_all_pre_targets :  $(PRE_TARGETS)
	@ if [ $(DEBUG) != 0 ]; then\
		echo "make: pre-targets are done in project '$(SNIFF_PROJECT)'";\
	fi

########################################################################
# symbolic_links
# 		create symbolic links to the object files in the 
#		shared object working environment and to the makefiles in the 
#		shared source working environment
########################################################################
symbolic_links :: check_dirs symbolic_links_to_targets symbolic_links_to_makefiles

 
########################################################################
# symbolic_link_to_dependencies_file
# 		create symbolic link to the dependencies file in the 
#		shared source working environment
########################################################################
symbolic_link_to_dependencies_file :: _symbolic_link_to_dependencies_file
_symbolic_link_to_dependencies_file :
	-@ echo "*** $(PARENT)";\
	_FILE=$(SNIFF_MAKEDIR)/$(SNIFF_DEPEND_INCL);\
	found=0;\
	if [ "$(SNIFF_ShSWS)" != "" ];then\
		for dir in _dummy_ `echo "$(SNIFF_ShSWS)"|sed -e 's/$(D)/ /g'`;do\
			if [ $$dir = "_dummy_" ]; then\
				continue;\
			fi;\
			if [ -r $$dir/$$_FILE ]; then\
				if [ $(DEBUG) != 0 ]; then\
					if [ "$(MK_SYMLINK)" != "" ]; then\
						echo make: Creating symbolic link to shared dependencies file:;\
					else\
						echo make: Copying shared dependencies file:;\
					fi;\
					echo "$$dir/$$_FILE";\
				fi;\
				$(RM) $$_FILE;\
				if [ "$(MK_SYMLINK)" != "" ]; then\
					$(MK_SYMLINK) $$dir/$$_FILE $$_FILE;\
				else\
					$(CP) -p $$dir/$$_FILE $$_FILE;\
				fi;\
				found=1;\
				break;\
			fi;\
		done;\
		if [ $$found = 0 -a $(DEBUG) != 0 ]; then\
			echo make: shared $$_FILE file not found;\
		fi;\
	fi

########################################################################
# clean_targets
# 		remove targets from the private working environment
########################################################################
clean_targets :: _clean_targets
_clean_targets :
	-@ echo "*** $(PARENT)";\
	echo make: Removing targets of the project \'$(SNIFF_PROJECT)\';\
	if [ "$(TARGET_DIR)" = "." ]; then\
		if [ $(DEBUG) != 0 ]; then\
			echo $(RM) $(SNIFF_TARGETS) \
				 $(SNIFF_TARGET).EXE *.mdp *.pdb;\
		fi;\
		if [ "$(SNIFF_TARGETS)" != "" ]; then\
			$(RM) $(SNIFF_TARGETS) \
				 $(SNIFF_TARGET).EXE *.mdp *.pdb;\
		fi;\
	else\
		if [ $(DEBUG) != 0 ]; then\
			echo $(RM) $(TARGET_DIR)/"*";\
		fi;\
		if [ "$(SNIFF_TARGETS)" != "" ]; then\
			$(RM) $(TARGET_DIR)/*;\
		fi;\
	fi 
########################################################################
# clean_objects
# 		remove object files  from the private working environment
########################################################################
clean_objects :: _clean_objects
_clean_objects :
	-@ echo "*** $(PARENT)";\
	echo make: Removing all objects of the project \'$(SNIFF_PROJECT)\';\
	if [ "$(OBJECT_DIR)" = "." ]; then\
		if [ $(DEBUG) != 0 ]; then\
			echo $(RM) $(OFILES) $(OTHER_OFILES);\
		fi;\
		if [ "$(OFILES)" != "" ]; then\
			$(RM) $(OFILES) $(OTHER_OFILES);\
		fi;\
	else\
		if [ $(DEBUG) != 0 ]; then\
			echo $(RM) $(OBJECT_DIR)/"*";\
		fi;\
		if [ "$(OFILES)" != "" ]; then\
			$(RM) $(OBJECT_DIR)/*;\
		fi;\
	fi 

########################################################################
# clean
# 		remove object files and targets from the private working environment
########################################################################
clean :: _clean
_clean :
	-@ echo "*** $(PARENT)";\
	echo make: Removing object files and targets of the project \'$(SNIFF_PROJECT)\';\
	if [ $(DEBUG) != 0 ]; then\
		echo $(RM) $(CLEANUP);\
	fi;\
	if [ "$(CLEANUP)" != "" ]; then\
		$(RM) $(CLEANUP);\
	fi

########################################################################
# sniffclean
# 		remove symbol files from the private working environment
########################################################################
sniffclean :: _sniffclean
_sniffclean :
	-@ echo "*** $(PARENT)";\
	echo make: Removing symbol files of the project \'$(SNIFF_PROJECT)\';\
	if [ $(DEBUG) != 0 ]; then\
		echo $(RM) $(SNIFF_SYMBOL_DIR)/.SniffDirInfo.dump $(SNIFF_SYMBOL_DIR)/$(SNIFF_PROJECT).symtab.*  $(SNIFF_SYMBOL_DIR)/*.lex.* $(SNIFF_SYMBOL_DIR)/*.ref.* $(SNIFF_SYMBOL_DIR)/*.index.* $(SNIFF_SYMBOL_DIR)/*.cpp $(SNIFF_SYMBOL_DIR)/*.cmi $(SNIFF_SYMBOL_DIR)/$(SNIFF_PROJECT).docstat $(SNIFF_SYMBOL_DIR)/*.state.*;\
	fi;\
	$(RM) $(SNIFF_SYMBOL_DIR)/.SniffDirInfo.dump $(SNIFF_SYMBOL_DIR)/$(SNIFF_PROJECT).symtab.*  $(SNIFF_SYMBOL_DIR)/*.lex.* $(SNIFF_SYMBOL_DIR)/*.ref.* $(SNIFF_SYMBOL_DIR)/*.index.* $(SNIFF_SYMBOL_DIR)/*.cpp $(SNIFF_SYMBOL_DIR)/*.cmi $(SNIFF_SYMBOL_DIR)/$(SNIFF_PROJECT).docstat $(SNIFF_SYMBOL_DIR)/*.state.*

########################################################################
# clean_state
# 		remove state files from the private working environment
########################################################################
clean_state :: _clean_state
_clean_state :
	-@ echo "*** $(PARENT)";\
	echo make: Removing state files of the project \'$(SNIFF_PROJECT)\';\
	if [ $(DEBUG) != 0 ]; then\
		echo $(RM) $(SNIFF_SYMBOL_DIR)/*.state2*;\
	fi;\
	$(RM) $(SNIFF_SYMBOL_DIR)/*.state2*
 
########################################################################
# clean_backup
# 		remove backup files from the private working environment
########################################################################
clean_backup :: _clean_backup
_clean_backup :
	-@ echo "*** $(PARENT)";\
	if [ "$(SNIFF_PROJECT)" != "" -a "$(SNIFF_FILETYPE_DIRS)" != "" ]; then\
		echo make: Removing backup files of the project \'$(SNIFF_PROJECT)\';\
		for dir in _dummy_ $(SNIFF_FILETYPE_DIRS); do\
			if [ $$dir = "_dummy_" ]; then\
				continue ;\
			fi;\
			if [ $(DEBUG) != 0 ]; then\
				echo $(RM) $$dir/*%;\
			fi;\
			$(RM) $$dir/*%;\
		done;\
	fi
 
########################################################################
# search_file
# 		search file in shared object directories
########################################################################
search_file :
	@dirs=`echo $(SNIFF_ShOWS)|sed -e 's/$(D)/ /g'`;\
	for dir in _dummy_ $$dirs; do\
		if [ $$dir = "_dummy_" ]; then\
			continue;\
		fi;\
		if [ $(DEBUG) != 0 ]; then\
			echo "make: searching in directory '$$dir'";\
		fi;\
		if [ -r $$dir/$(FILE) ]; then\
			echo make: $(FILE) found in the shared object directory;\
			echo "      '$$dir'";\
			exit;\
		fi;\
	done;\
	echo make: $(FILE) not found
 
########################################################################
# secondary targets
# 		help the targets above to do their job
########################################################################
symbolic_links_to_targets :
	-@ echo "*** $(PARENT)";\
	if [ "$(OFILES)$(OTHER_OFILES)$(SNIFF_TARGETS)$(SNIFF_DERIVED_FILES)" != "" -a "$(SNIFF_ShOWS)" != "" ];then\
		if [ "$(MK_SYMLINK)" != "" ]; then\
			echo "make: creating symbolic links to shared object files and targets in project '$(SNIFF_PROJECT)'";\
		else\
			echo "make: copying shared object files and targets in project '$(SNIFF_PROJECT)'";\
		fi;\
		for o in _dummy_ $(OFILES) $(OTHER_OFILES) $(SNIFF_TARGETS) $(TARGET_DIR)/$(SNIFF_TARGET).EXE $(SNIFF_DERIVED_FILES);do\
			if [ "$$o" != "_dummy_" -a ! -r ./$$o ];then\
				for dir in _dummy_ `echo "$(SNIFF_ShOWS)"|sed -e 's/$(D)/ /g'`;do\
					if [ $$dir = "_dummy_" ]; then\
						continue;\
					fi;\
					if [ -r $$dir/$$o ]; then\
						if [ $(DEBUG) != 0 ]; then\
							if [ "$(MK_SYMLINK)" != "" ]; then\
								echo $(MK_SYMLINK) $$dir/$$o ./$$o;\
							else\
								echo $(CP) -p $$dir/$$o ./$$o;\
							fi;\
						fi;\
						if [ "$(MK_SYMLINK)" != "" ]; then\
							$(MK_SYMLINK) $$dir/$$o ./$$o;\
						else\
							$(CP) -p $$dir/$$o ./$$o;\
						fi;\
						break;\
					fi;\
				done;\
			fi;\
		done;\
	fi
 
symbolic_links_to_makefiles :
	-@if [ "$(SNIFF_Make_FILES)" != "" -a "$(SNIFF_ShSWS)" != "" ];then\
		if [ "$(MK_SYMLINK)" != "" ]; then\
			echo make: Creating symbolic links to shared make files;\
		else\
			echo make: Copying shared make files;\
		fi;\
		for f in _dummy_ $(SNIFF_Make_FILES);do\
			if [ ! -f $$f ];then\
				for dir in _dummy_ `echo "$(SNIFF_ShSWS)"|sed -e 's/$(D)/ /g'`;do\
					if [ $$dir = "_dummy_" ]; then\
						continue;\
					fi;\
					if [ -r $$dir/$$f ]; then\
						if [ $(DEBUG) != 0 ]; then\
							if [ "$(MK_SYMLINK)" != "" ]; then\
								echo $(MK_SYMLINK) $$dir/$$f $$f;\
							else\
								echo $(CP) -p $$dir/$$f $$f;\
							fi;\
						fi;\
						if [ "$(MK_SYMLINK)" != "" ]; then\
							$(MK_SYMLINK) $$dir/$$f $$f;\
						else\
							$(CP) -p $$dir/$$f $$f;\
						fi;\
						break;\
					fi;\
				done;\
			fi;\
		done;\
	fi

echo_parent :
	-@ echo "*** $(PARENT)"
	
check_vpath :
	-@ echo "================================";\
	echo "check_vpath not required anymore";\
	echo "================================"

########################################################################
# help
# 		print help message
########################################################################
help :
	@echo "";\
	echo "SNiFF+ V3.0 make support";\
	echo "";\
	echo "Default target:";\
	echo "---------------";\
	echo "  [all] [DEBUG=1] [CHECK_UPDATE=0]";\
	echo "          Make default target recursively.";\
	echo "";\
	echo "Help targets:";\
	echo "-------------";\
	echo "  pre_targets [DEBUG=1] [CHECK_UPDATE=0]";\
	echo "          Make pre-targets (for example compile IDL files).";\
	echo "";\
	echo "  search_file FILE=<filename>";\
	echo "          Search file in shared object directories.";\
	echo "";\
	echo "Recursively help targets:";\
	echo "-------------------------";\
	echo "  symbolic_links [DEBUG=1]";\
	echo "          Create symbolic links to the object files in the shared object";\
	echo "          working environment and to the makefiles in the shared source working environment.";\
	echo "";\
	echo "  clean [DEBUG=1]";\
	echo "          Remove object files, java class files and targets from the working environment.";\
	echo "";\
	echo "  all_pre_targets [DEBUG=1] [CHECK_UPDATE=0]";\
	echo "          Make pre-targets recursively (for example compile IDL files).";\
	echo "";\
	echo "  symbolic_link_to_dependencies_file [DEBUG=1]";\
	echo "          Create symbolic link to the dependencies file in the shared";\
	echo "          source working environment.";\
	echo "";\
	echo "  sniffclean [DEBUG=1]";\
	echo "          Remove symbol files of SNiFF+ from the generate directory in";\
	echo "          working environment.";\
	echo "";\
	echo "  clean_targets [DEBUG=1]";\
	echo "          Remove targets from the working environment.";\
	echo "";\
	echo "  clean_objects [DEBUG=1]";\
	echo "          Remove all objects from the working environment.";\
	echo "";\
	echo "  clean_state [DEBUG=1]";\
	echo "          Remove state files from the generate directory in working environment.";\
	echo "";\
	echo "  clean_backup [DEBUG=1]";\
	echo "          Remove backup files from the working environment.";\
	echo "";\
	echo "Parameters:";\
	echo "-----------";\
	echo "  CHECK_UPDATE=0         don't check the up-to-date status of working environment";\
	echo "  DEBUG=1                print out control messages"; \
	echo "  FORCE=1                always builds targets"; \
	echo "";
 
#        Includes the include.incl generated make support file.
#        The file includes the needed parts of general.mk for the current project.
include $(SNIFF_MAKEDIR)/$(SNIFF_INCLUDE_INCL)
 
#        Includes the obsolete rules of general.mk
#include $(SNIFF_DIR)/make_support/general.obsolete.mk
 
#        Platform specific make macros
include $(SNIFF_DIR)/make_support/$(PLATFORM).mk

#	Nothing matched the ofile to be built, so we try to make it in a subdir
#	(if target contains a subdir) or print error
%.o %.res: $(FORCE_RULE)
	@d=`dirname '$@'`;\
	relative=`echo $$d | cut -d"/" -f1`;\
	if [ "$$d" != "." -a "$$d" != "$(OBJECT_DIR)" -a "$$relative" != "" ]; then\
		dir=`echo $$d | sed -e 's#/$(OBJECT_DIR)$$##g'`;\
		cd $$dir;\
		echo make: Changing directory to \'$$dir\';\
		$(OWN_MAKE) -f $(MAKEFILE) $(OBJECT_DIR)/`basename '$@'`;\
	else\
		echo make: Don\'t known how to make $@;\
	fi;

#        Java specific rules
#        These rules might be called from general.java.mk, if a non Java project is a subproject of a Java project.
#        These rules can be overloaded for specific languages, e.g. for C++, if JNI generation should be handled.

#        make all targets recursively in a Java project should make all targets for non Java projects as well
targets_all : all
	@if [ $(DEBUG) != 0 ]; then\
		echo "make: all targets are done in project '$(SNIFF_PROJECT)'";\
	fi
