#
#	general.mk
#
#	SNiFF+ V3.0
#
#	Common make rules for yacc and lex.
#
#	Copyright (C) 1998 TakeFive Software GmbH
#	Salzburg, Austria (e-mail: sniff@TakeFive.co.at)
#
#	Author: Sandor Korosi

#        Set default value for renaming generated y.tab.h to basename.h.
YACC_MOVE_HEADER = 1
CXX_OUTPUT_FLAG		= -o $@
CC_OUTPUT_FLAG		= -o $@

#        All yacc compiler flags and yacc command
ALL_YACCFLAGS = $(OTHER_YACCFLAGS) $(COMMON_YACCFLAGS) $(OS_YACCFLAGS) $(YACCFLAGS)
YACC_CMD = $(YACC) $(ALL_YACCFLAGS) `basename $<`

#        C++ compiler command for generated files of yacc
YACC_CXX_CMD = $(CXX) $(ALL_CXXFLAGS) $(OVERALL_OPTION_CXX)  $(SNIFF_Yacc_Source_DIR)/$*.C $(CXX_OUTPUT_FLAG)
#        C compiler command for generated files of yacc
YACC_CC_CMD = $(CC) $(ALL_CFLAGS) $(OVERALL_OPTION_CC)  $(SNIFF_Yacc_Source_DIR)/$*.c $(CC_OUTPUT_FLAG)

#        All lex flags and lex command
ALL_LEXFLAGS = $(OTHER_LEXFLAGS) $(COMMON_LEXFLAGS) $(OS_LEXFLAGS) $(LEXFLAGS)
LEX_CMD = $(LEX) $(ALL_LEXFLAGS) -o`basename $*.$(LEX_CFILE_SUFFIX)` `basename $<`

#        C++ compiler command for generated files of lex
LEX_CXX_CMD = $(CXX) $(ALL_CXXFLAGS) $(OVERALL_OPTION_CXX)  $(SNIFF_Lex_Source_DIR)/$*.C $(CXX_OUTPUT_FLAG)
#        C compiler command for generated files of lex
LEX_CC_CMD = $(CC) $(ALL_CFLAGS) $(OVERALL_OPTION_CC)  $(SNIFF_Lex_Source_DIR)/$*.c $(CC_OUTPUT_FLAG)
 
## ATTENTION Dont remove the ;  after the following rule (empty rule)
#%.o : $(OBJECT_DIR)/%.o ; 

#       rules for yacc
$(OBJECT_DIR)/%.o : $(SNIFF_Yacc_Source_DIR)/%.y $(FORCE_RULE)
	@ d=`dirname '$@'`;\
	relative=`echo $$d | cut -d"/" -f1`;\
	if [ "$$d" != "." -a "$$d" != "$(OBJECT_DIR)" -a "$$relative" != "" ]; then\
		cd $$d;\
		echo make: Changing directory to \'$$d\';\
		$(OWN_MAKE) -f $(MAKEFILE) `basename '$@'`;\
	else\
		$(OWN_MAKE) -f $(MAKEFILE) $(SNIFF_Yacc_Source_DIR)/$*.$(YACC_CFILE_SUFFIX);\
		if [ "$(YACC_CFILE_SUFFIX)" = "C" ]; then \
			TARGET=$@; TARGET_TYPE="C++ object"; $(MAKE_TARGET);\
			if [ "$(DEBUG)" = "1" ]; then \
				echo "$(YACC_CXX_CMD)";\
			fi;\
			$(YACC_CXX_CMD);\
		else\
			TARGET=$@; TARGET_TYPE="C object"; $(MAKE_TARGET);\
			if [ "$(DEBUG)" = "1" ]; then \
				echo "$(YACC_CC_CMD)";\
			fi;\
			$(YACC_CC_CMD);\
		fi;\
	fi;

$(SNIFF_Yacc_Source_DIR)/%.C : $(SNIFF_Yacc_Source_DIR)/%.y $(FORCE_RULE)
	@ $(OWN_MAKE) -f $(MAKEFILE) `basename $*.y_`

$(SNIFF_Yacc_Source_DIR)/%.c : $(SNIFF_Yacc_Source_DIR)/%.y $(FORCE_RULE)
	@ $(OWN_MAKE) -f $(MAKEFILE) `basename $*.y_`

%.y_ : $(SNIFF_Yacc_Source_DIR)/%.y
	@ cd $(SNIFF_Yacc_Source_DIR); \
	TARGET=y.tab.c; TARGET_TYPE="yacc generated"; $(MAKE_TARGET);\
	if [ "$(DEBUG)" = "1" ]; then \
		echo "$(YACC_CMD)";\
	fi;\
	$(YACC_CMD);\
	ret=$$?;\
	echo make: move y.tab.c to `basename $*.$(YACC_CFILE_SUFFIX)`;\
	$(RM) `basename $*.$(YACC_CFILE_SUFFIX)`;\
	$(MV) y.tab.c `basename $*.$(YACC_CFILE_SUFFIX)`;\
	if [ "$(YACC_MOVE_HEADER)" = "1" ]; then \
		echo make: move y.tab.h to `basename $*.h`;\
		$(RM) `basename $*.h`;\
		$(MV) y.tab.h `basename $*.h`;\
	fi;\
	exit $$ret

#       rules for lex
$(OBJECT_DIR)/%.o : $(SNIFF_Lex_Source_DIR)/%.l $(FORCE_RULE)
	@ d=`dirname '$@'`;\
	relative=`echo $$d | cut -d"/" -f1`;\
	if [ "$$d" != "." -a "$$d" != "$(OBJECT_DIR)" -a "$$relative" != "" ]; then\
		cd $$d;\
		echo make: Changing directory to \'$$d\';\
		$(OWN_MAKE) -f $(MAKEFILE) `basename '$@'`;\
	else\
		$(OWN_MAKE) -f $(MAKEFILE) $(SNIFF_Lex_Source_DIR)/$*.$(LEX_CFILE_SUFFIX);\
		if [ "$(LEX_CFILE_SUFFIX)" = "C" ]; then \
			TARGET=$@; TARGET_TYPE="C++ object"; $(MAKE_TARGET);\
			if [ "$(DEBUG)" = "1" ]; then \
				echo "$(LEX_CXX_CMD)";\
			fi;\
			$(LEX_CXX_CMD);\
		else\
			TARGET=$@; TARGET_TYPE="C object"; $(MAKE_TARGET);\
			if [ "$(DEBUG)" = "1" ]; then \
				echo "$(LEX_CC_CMD)";\
		fi;\
			$(LEX_CC_CMD);\
		fi;\
	fi;

$(SNIFF_Lex_Source_DIR)/%.C : $(SNIFF_Lex_Source_DIR)/%.l $(FORCE_RULE)
	@ $(OWN_MAKE) -f $(MAKEFILE) `basename $*.l_`

$(SNIFF_Lex_Source_DIR)/%.c : $(SNIFF_Lex_Source_DIR)/%.l $(FORCE_RULE)
	@ $(OWN_MAKE) -f $(MAKEFILE) `basename $*.l_`

%.l_ : $(SNIFF_Lex_Source_DIR)/%.l
	@ cd $(SNIFF_Lex_Source_DIR); \
	TARGET=`basename $*.$(LEX_CFILE_SUFFIX)`; TARGET_TYPE="lex generated"; $(MAKE_TARGET);\
	if [ "$(DEBUG)" = "1" ]; then \
		echo "$(LEX_CMD)";\
	fi;\
	$(LEX_CMD)
